package de.ipk.pdwexpress.peObjGenerator;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.dbutils.DbUtils;
import org.apache.log4j.Logger;

import de.ipk.pdwexpress.tableObjects.ExternDB;
import de.ipk.pdwexpress.tableObjects.Object_Type;
import de.ipk.pdwexpress.tableObjects.TRM_Category;


/**
 * 
 * Class generates files within package 'de.ipk.pdwexpress.peobjects'
 * 
 * @author Thomas
 *
 */
public class PEObjectGeneratorExec {
	
	protected Logger logger;
	
	protected final Connection con;
	protected final PEObjectGenerator generatorOBJ = new PEObjectGenerator();
	protected final ExternDBsGenerator generatorEDB = new ExternDBsGenerator();
	
	/**
	 * 
	 * @param con Connection to Oracle PDWExpress Database 
	 */
	public PEObjectGeneratorExec(Connection con){
		this.con=con;
		logger=Logger.getLogger(this.getClass());
	}

	
	/**
	 * Creates directorys 'de/ipk/pdwexpress/peobjects' within destination path <br>
	 * 
	 * generates classes <br>
	 * 
	 * @param args destination path for generated classes
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 * @throws IOException
	 */
	public static void main(String[] args) throws ClassNotFoundException, SQLException, IOException {
		
//		BasicConfigurator.configure();
		
		// Destination path
		String pathRoot =null;
		if (args.length == 0 || args[0] ==null){
			System.out.println("parameter should be destination path");
			System.exit(-1);
		}
		else
			pathRoot =args[0];
		
		File path =new File(pathRoot,"de/ipk/pdwexpress/peobjects");
		path.mkdirs();
//		FileUtils.cleanDirectory(path);
		
        Class.forName("oracle.jdbc.driver.OracleDriver");
        Connection con = DriverManager.getConnection("jdbc:oracle:thin:@pgrc-16.ipk-gatersleben.de:1521:pdw","pdw_express_stage","");
		
		PEObjectGeneratorExec exec = new PEObjectGeneratorExec(con);
		exec.writeFiles(path);
		
		DbUtils.closeQuietly(con);
		
		System.out.println("finished.");
		

	}
	
	
	/**
	 * Generates classes with directory 
	 * (does not create subfolder for de/ipk/....)
	 * 
	 * @param path The destination path.
	 * @throws SQLException
	 * @throws IOException
	 */
	public void writeFiles(File path) throws SQLException, IOException{
		
		
		 Set<ObjectTypes_Categories> allObtCat=readObjectType_Cat();
		 Set<ExternDB> allEdb = readEDB();
		 
		 String fileName ;
		 File destFile;
//		 PDWExpressDB pdwExpress = new PDWExpressDB(allObtCat,edb);

		 // Generate ExternDB
		 fileName = "PE_ExternDBEnum.java";
		 destFile = new File(path,fileName);
		 logger.info("write file "+destFile);
		 
		 generateEDB(destFile, allEdb);
		 
		 // Generate Files based on OBJECT_TYPES
		 for (ObjectTypes_Categories ojtCat:allObtCat){
			 
			 fileName = "PE_"+ojtCat.getObjectType().getOjt_name()+".java";
			 destFile = new File(path,fileName);
			 logger.info("write file "+destFile);
			 
			 generateOBJ(destFile, ojtCat);
		 }
		 
		
	}
	


	protected void generateEDB(File destFile, Set<ExternDB> allEdb) throws IOException{
		FileWriter writer = new FileWriter(destFile);
		writer.write(generatorEDB.generate(allEdb));
		writer.close();		
	}


	/**
	 * 
	 * @param destFile
	 * @param objCat
	 * @throws IOException
	 */
	protected void generateOBJ(File destFile,ObjectTypes_Categories objCat) throws IOException{
		FileWriter writer = new FileWriter(destFile);
		writer.write(generatorOBJ.generate(objCat));
		writer.close();
	}
	
	/**
	 * 
	 * @return
	 * @throws SQLException
	 */
	protected Set<ObjectTypes_Categories> readObjectType_Cat() throws SQLException {
		
		logger.debug("starting Methode: readObjectType_Cat");
		
		Set<TRM_Category> allCommonCategories=readCommonCategories();
		
		Set<ObjectTypes_Categories> allObtCat = new HashSet<ObjectTypes_Categories>();
		
		String query="select ojt_id, ojt_name, OJT_ABBREVIATION from object_types_t";
		
		
		Statement stmtOjt = con.createStatement();
		ResultSet rs = stmtOjt.executeQuery(query);
		
		Object_Type ojt = null;
		while (rs.next()){
			
			ojt = new Object_Type(rs.getString(1),rs.getString(2),rs.getString(3));
									
			ObjectTypes_Categories obtCat = new ObjectTypes_Categories(ojt);
			obtCat.setCategoriesAttr(readCategoriesOfOJT(ojt.getOjt_name()));
			obtCat.setCategoriesCommon(allCommonCategories);
			obtCat.setCategoriesFK(readFKCategories(ojt.getObt_abbreviation()));
			
			allObtCat.add(obtCat);
		}
		
		logger.info(allObtCat.size() +" Object types found");

		
		DbUtils.closeQuietly(rs);
		DbUtils.closeQuietly(stmtOjt);
		
		return allObtCat;
	}
	
	
	/**
	 * 
	 * @return
	 * @throws SQLException
	 */
	protected Set<TRM_Category> readCommonCategories() throws SQLException {
		
		logger.debug("starting Methode: readCommonCategories");
		Set<TRM_Category> allCommonCategories=new HashSet<TRM_Category>();
		
		String query="SELECT TCT_ID,TCT_NAME, TCT_DESC, tct_many " +
						"FROM " +
						"TRM_CATEGORIES_T  " +
						"where level > 1 " +
						"CONNECT BY PRIOR tct_id=tct_tct_id " +
						"START WITH tct_id = (	SELECT tct_id FROM TRM_CATEGORIES_T WHERE tct_name='OBJECT_ATTR') ";
				
		
		Statement stmtOjt = con.createStatement();
		ResultSet rs = stmtOjt.executeQuery(query);
		
		TRM_Category tct = null;		
		while (rs.next()){			
			tct = new TRM_Category(rs.getString(1),rs.getString(2),rs.getString(3),rs.getString(4));
			allCommonCategories.add(tct);
		}
		
		logger.info("readCommonCategories "+allCommonCategories.size() +" Common Categories found");

		
		DbUtils.closeQuietly(rs);
		DbUtils.closeQuietly(stmtOjt);
		
		return allCommonCategories;
	}
		
	
	/**
	 * 
	 * @return
	 * @throws SQLException
	 */
	protected Set<TRM_Category> readFKCategories(String ojt_abbreviation) throws SQLException {
		
		logger.debug("starting Methode: readFKCategories("+ojt_abbreviation+")");
		Set<TRM_Category> allFKCategories=new HashSet<TRM_Category>();
		
		String query="SELECT TCT_ID,TCT_NAME,TCT_DESC, tct_many " +
						"FROM " +
						"TRM_CATEGORIES_T  " +
						"where level > 1 " +
						"and substr(TCT_NAME,1,4 ) ='"+ojt_abbreviation+"_' " +
						"CONNECT BY PRIOR tct_id=tct_tct_id " +
						"START WITH tct_id = (	SELECT tct_id FROM TRM_CATEGORIES_T WHERE tct_name='FOREIGNKEYS') ";
				
		
		Statement stmtOjt = con.createStatement();
		ResultSet rs = stmtOjt.executeQuery(query);
		
		TRM_Category tct = null;		
		while (rs.next()){			
			tct = new TRM_Category(rs.getString(1),rs.getString(2),rs.getString(3),rs.getString(4));
			allFKCategories.add(tct);
		}
		
		logger.info("readFKCategories: "+allFKCategories.size() +" Foreign Keys found for "+ojt_abbreviation);

		
		DbUtils.closeQuietly(rs);
		DbUtils.closeQuietly(stmtOjt);
		
		return allFKCategories;
	}	
	
	

	/**
	 * 
	 * @param ojt_name
	 * @return never null
	 * @throws SQLException
	 */
	private Set<TRM_Category> readCategoriesOfOJT(String ojt_name) throws SQLException{
		
		String query =""+
		"SELECT "+
		"  tct_id,TCT_NAME,TCT_DESC, TCT_MANY "+
		"FROM "+
		"	TRM_CATEGORIES_T  " +
		"where level >1 "+
		"CONNECT BY PRIOR tct_id=tct_tct_id "+
		"	START WITH tct_id = (	SELECT "+
		"								ojt_tct_id "+ 
		"							FROM "+
		"								OBJECT_TYPES_T "+ 
		"							WHERE "+
		"								ojt_name='"+ojt_name+"')";
		
		logger.debug(query);
		Statement stmtCat = con.createStatement();
		
		ResultSet rs = stmtCat.executeQuery(query);
		
		Set<TRM_Category> categories = new HashSet<TRM_Category>();
		while(rs.next()){
			TRM_Category cat = new TRM_Category(rs.getString(1),rs.getString(2),rs.getString(3),rs.getString(4));
			categories.add(cat);
		}
		
		
		logger.info(categories.size()+" categories found for "+ojt_name);
		
		
		DbUtils.closeQuietly(rs);
		DbUtils.closeQuietly(stmtCat);
		
		return categories;
	}
	
	
	private Set<ExternDB> readEDB() throws SQLException {
		
		Set<ExternDB> allEdb = new HashSet<ExternDB>();
		PreparedStatement stmt=null;
		
		try {
			stmt = con.prepareStatement("select edb_id, edb_name, edb_url, edb_desc from externdb_t");
			
			ResultSet rs =stmt.executeQuery();
			
			while(rs.next()){
				ExternDB edb = new ExternDB(rs.getString(1), rs.getString(2), rs.getString(3), rs.getString(4));
				allEdb.add(edb);
			}
			
		} catch (SQLException ex) {
			throw(ex);
		}finally{
			DbUtils.closeQuietly(stmt);
		}
		
		
		return allEdb;
	}	

}
