package de.ipk.pdwexpress.upload;

import java.sql.SQLException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.commons.collections.map.MultiValueMap;
import org.apache.log4j.Logger;

import de.ipk.pdwexpress.tableObjects.Term;

public abstract class AbstractPEObject implements PE_Object {
	
	private static ImportHandler importHandler;
	
	private String obj_id;
	
	protected Logger logger;
	
//	private final Map<String,Term>  catTermsMap=new HashMap<String,Term> ();
//	private final MultiValueMap  catTermsMap=new MultiValueMap ();
	private final MultiValueMap  catTermsMap=MultiValueMap.decorate(new HashMap()) ;
	
	
	
	
	public AbstractPEObject(){
		logger = Logger.getLogger(this.getClass());
	}
	
	/**
	 * Checks if the dataset already exists <br>
	 * (Datasets are equals if OBJ_Identifier, TRM_EDBACC and TRM_EDB_ID 
	 * are equals) <br>
	 * <br> 
	 * Stores the data in in the database, if not exists <br>
	 * 
	 * @return OBJ_ID from table OBJECTS_T
	 * @throws SQLException
	 */
	public String saveObject() throws SQLException{
		
		if (importHandler == null){
			throw new NullPointerException("Set the ImportHandler first ! - AbstractPEObject. setImportHandler()");
		}
		
		if (importHandler.objectExists(this)){
//			logger.warn("Object not saved. object:"+this);
			return this.getObj_id();
		}
		
		// insert into OBJECTS_T
		importHandler.insertObject(this);
		
		// Loop: insert Terms and Properties		
		for( Object key  : catTermsMap.keySet()){
			
			Collection<Term> allTerms =  (Collection<Term>) catTermsMap.get(key);
			
			for(Term term: allTerms){
				importHandler.insertObjProperty(term, this);
			}
		}
		
		logger.debug("Object saved. object:"+this);
		
		return this.getObj_id();
	}

	public static ImportHandler getImportHandler() {
		return importHandler;
	}

	public static void setImportHandler(ImportHandler importHandler) {
		AbstractPEObject.importHandler = importHandler;
	}

	/**
	 * @see de.ipk.pdwexpress.upload.PE_Object#getObj_id()
	 */
	public String getObj_id() {
		return obj_id;
	}
	
	
	public abstract Term getOBJ_Identifier_Term();
	
	/**
	 * Removes leading and ending spaces from value and acc 
	 * @param term
	 * @return The chaged Parameter term.
	 */
	private Term cleanTerm(Term term){
		String acc =term.getTrm_edbacc();
		String value =term.getTrm_value();
		
		if (acc != null)
			term.setTrm_edbacc(acc.trim());
		
		if (value != null)
			term.setTrm_value(value.trim());
		
		return term;
	}
	
	/**
	 * Assign a Term to the category. <br>
	 * Existing Terms associated with the category are <b>removed</b> ! <br>
	 * Use addTerm if existting terms should not be removed.
	 * 
	 * @param tct_id
	 * @param term
	 * 
	 * @throws NullPointerException If term is null.
	 */
	protected final void setTerm(String tct_id, Term term){
		
		if (logger.isDebugEnabled())
			logger.debug("Set Term. TCT_ID:"+tct_id+" TERM:"+term);

		// remove all Terms
		Collection<Term> curContent = catTermsMap.getCollection(tct_id);
		if (curContent != null)
			curContent.clear();
		
		if(term.getTrm_value() == null){
			logger.warn("Term for category "+tct_id+" has trm_value NULL. Term ignored");
			return;
		}else if(term.getTrm_value().trim().length()== 0){
			logger.warn("Term for category "+tct_id+" has length 0. Term ignored");
			return;			
		}		
		
		
		
		catTermsMap.put(tct_id, cleanTerm(term));
	}

	/**
	 * AAssign a Term to the category. <br>
	 * 
	 * If <code>term.getTrm_value()</code> returns NULL or <code> term.getTrm_value().trim().length()== 0 </code> the the call is ignored.
	 * @param tct_id Database Category ID (key)
	 * @param term The Term to add (value) 
	 * 
	 * @throws NullPointerException If term is null.
	 */
	protected final void addTerm(String tct_id, Term term){
		
		if(term.getTrm_value() == null){
			logger.warn("Term for category "+tct_id+" has trm_value NULL. Term ignored");
			return;
		}else if(term.getTrm_value().trim().length()== 0){
			logger.warn("Term for category "+tct_id+" has length 0. Term ignored");
			return;			
		}
		
		if (logger.isDebugEnabled())
			logger.debug("Adding Term. TCT_ID:"+tct_id+" TERM:"+term);
		
		catTermsMap.put(tct_id, cleanTerm(term));
	}
	
	/**
	 * 
	 * get Term from the intern Map
	 * 
	 * @param tct_id Database Category ID (key)
	 * @return The Term assiciated with te tct_id. NULL if there is no mapping.
	 */
	protected final Term getTerm(String tct_id){
		
		Collection<Term> col =  catTermsMap.getCollection(tct_id);
		
		if (col!=null){
		
			if (col.size() > 1)
				logger.warn("getTerm: ONE value for catogory requestd. But there are more than one value assigned. catogory:"+tct_id+" count:"+col.size());
			
			Iterator<Term> it = col.iterator();
			if(it.hasNext())
				return it.next();
		}
		
		return null;
		
//		return catTermsMap.get(tct_id);
	}

	/**
	 * get Term from the intern Map
	 * 
	 * @param tct_id Database Category ID (key)
	 * @return The Term assiciated with te tct_id. NULL if there is no mapping.
	 */
	protected final Collection<Term> getAllTerms(String tct_id){
		
		Collection<Term> col =  catTermsMap.getCollection(tct_id);
		
		return col;
		
//		return catTermsMap.get(tct_id);
	}	
	

	
	void setObj_id(String obj_id) {
		this.obj_id = obj_id;
	}

	private final static String S1="OBJ_ID:";
	private final static String S2=" OBJ_Identifier:";
	private final static String S3=" OBJ_Name:";
	private final static String S4="null";

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(S1+getObj_id());
		sb.append(S2+getOBJ_Identifier());
		sb.append(S3+getOBJ_Name());
		return sb.toString()+super.toString();
	}
	
//	public  void setOBJ_Description(String value){}
	
	

}
