package de.ipk.pdwexpress.upload;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import oracle.jdbc.OracleResultSet;

import org.apache.commons.dbutils.DbUtils;
import org.apache.log4j.Logger;

import de.ipk.pdwexpress.tableObjects.EditingSession;
import de.ipk.pdwexpress.tableObjects.Term;

public class ImportHandler {
	
	protected Logger logger;
	private final EditingSession ese;
	private final String edb_id_unref="0";
	private Connection con;
	private boolean killed=false;
	private boolean closed=false;
	
//	/**
//	 * Parameters: <br>
//	 * 1 trm_tct_id <br>
//	 * 2 TRM_VALUE <br>
//	 * --3 TRM_EDBACC <br>
//	 * --4 TRM_EDB_ID  <br>
//	 */
//	private PreparedStatement stmtQrTerm;
//	static final String sqlQrTerm=	"select trm_id from terms_T "+
//									"where trm_tct_id = ?  "+
//									"and TRM_VALUE = ? "+
//									"and TRM_EDBACC is null "+
//									"and TRM_EDB_ID is null "+
//									" ";

	/**
	 * Parameters: <br>
	 * 1 trm_tct_id <br>
	 * 2 TRM_VALUE <br>
	 * 3 TRM_EDBACC <br>
	 * 4 TRM_EDB_ID  <br>
	 */
	private PreparedStatement stmtQrTermEDB;
	static final String sqlQrTermEDB=	"select trm_id from terms_T "+
									"where trm_tct_id = ?  "+
									"and TRM_VALUE = ? "+
									"and TRM_EDBACC = ? "+
									"and TRM_EDB_ID = ? "+
									" ";
	
	/**
	 * Parameters: <br>
	 * 1 obj_ojt_id  <br>
	 * 2 obj_ese_id
	 * 
	 */
	private PreparedStatement stmtInObject;
	static final String sqlInObject="insert into OBJECTS_T (obj_ojt_id,obj_ese_id) values (?,?)";

	/**
	 * Parameters: <br>
	 * 1 TRM_TCT_ID <br>
	 * 2 TRM_VALUE <br>
	 * 3 TRM_DESC <br>
	 * 4 TRM_EDBACC <br>
	 * 5 TRM_EDB_ID <br>
	 * 6 trm_ese_id <br>
	 */
	private PreparedStatement stmtInTerm;
	static final String sqlInTerm="	insert into terms_t " +
									"( TRM_TCT_ID, TRM_VALUE, TRM_DESC, TRM_EDBACC, TRM_EDB_ID, trm_ese_id)" +
									"values (?,?,?,?,?,?)";

	/**
	 * Parameters: <br>
	 *  1 OPR_TRM_ID <br>
	 *  2 OPR_OBJ_ID <br> 
	 *  3 opr_ese_id
	 */
	private PreparedStatement stmtInObjProp;
	static final String sqlInObjProp=	"insert into OBJ_PROPERTIES_T "+
										" ( OPR_TRM_ID, OPR_OBJ_ID, opr_ese_id  ) "+
										" values (?,?,?) ";
	
	/**
	 * Parameters: <br>
	 * 1 TRM_VALUE  <br> 
	 * 2 TRM_EDBACC <br>
	 * 3 TRM_EDB_ID <br>
	 */
	private PreparedStatement stmtQrEquealsObj;
	static final String sqlQrEquealsObj="SELECT  " +
			"	opr_obj_id  " +
			"FROM  " +
			"	TERMS_T  " +
			"join OBJ_PROPERTIES_T on trm_id = opr_trm_id  " +
			"where  " +
			"TRM_TCT_ID = (select TCT_ID " +
			"				from TRM_CATEGORIES_T " +
			"				where tct_name ='OBJ_Identifier')  " +
			"and TRM_VALUE =  ? " +			
			"and TRM_EDBACC = ? " +
			"and TRM_EDB_ID = ? "; 

//	private ImportHandler(){
//		logger = Logger.getLogger(this.getClass());
//		
//		// get unref DB
//	}
//	
//	/**
//	 * Stores Data in database
//	 * 
//	 * @param con The connection to PDWExpressStage.
//	 * @throws SQLException
//	 */
//	private ImportHandler(Connection con) throws SQLException{
//		this();
//		this.con = con;
//		insertEditingSession(ese);
//		prepareStatements();
//	}
	
	/**
	 * 
	 * @param con The connection to the database.
	 * @param ese Description of impert
	 * @throws SQLException
	 */
	public ImportHandler(Connection con, EditingSession ese) throws SQLException{
		logger = Logger.getLogger(this.getClass());
		this.con = con;
		this.ese=ese;
		insertEditingSession(ese);
		prepareStatements();
	}

	/**
	 * 
	 * @return The ID of the current ImportSession
	 */
	public String getEditingSessionID(){
		return ese.getEse_Id();
	}
	
	
	
	/**
	 * 
	 * @param obj The Object that should be aved.
	 * @return  The ID of the Object within the database
	 * @throws SQLException
	 */
	public String insertObject(AbstractPEObject obj) throws SQLException{
		
		logger.info("insertObject: "+obj);

		// insert values und execute stmt
		stmtInObject.setString(1, obj.getObj_ojt_id());
		stmtInObject.executeUpdate();
		
				
		// get the new primary key
		OracleResultSet rsKey = (OracleResultSet) stmtInObject.getGeneratedKeys();

		if (rsKey.next())
			obj.setObj_id(rsKey.getString(1));
		else{
			logger.fatal("insertObject: No generated primary key found");
			throw new SQLException("No generated primary key found");
		}
		
		logger.debug("new OBJ_ID:"+obj.getObj_id());

		// close rs and clear stmt
		DbUtils.closeQuietly(rsKey);
//		stmtInObject.clearParameters();
		
		return obj.getObj_id();
	}
	
	
	/**
	 * Checks if the Term already exists in the Database
	 * if not insert Term
	 * 
	 * Set the TRM_ID of the Parameter term
	 * @param term The Term
	 * @return the TRM_ID (database ID)
	 * @throws SQLException 
	 */
	protected String synchronizeTerm(Term term) throws SQLException{
		
		
		ResultSet rs = null;
		try{
			
			// Search for Term in DB ...

			if (term.getTrm_edbacc() == null && term.getTrm_edb_id() == null){
//				logger.debug("searching for Term "+term+" without db acc ");
				
				term.setTrm_edbacc("unreferenced");
				term.setTrm_edb_id(edb_id_unref);
//				stmtQrTerm.setObject(1, term.getTrm_tct_id());
//				stmtQrTerm.setObject(2, term.getTrm_value());
//				rs = stmtQrTerm.executeQuery();	
				
			}
//			else{
				logger.debug("searching for Term "+term);
				
				stmtQrTermEDB.setObject(1, term.getTrm_tct_id());
				stmtQrTermEDB.setObject(2, term.getTrm_value());  
				stmtQrTermEDB.setObject(3, term.getTrm_edbacc());
				stmtQrTermEDB.setObject(4, term.getTrm_edb_id());
				rs = stmtQrTermEDB.executeQuery();				
//			}
			
			
			if (rs.next()){
			  // ... if exists get TRM_ID	
			  term.setTrm_id( rs.getString(1) );	
			  logger.debug("Term found:" +term);
			}else{
				
				logger.debug("Term not found");
				// ... ResultSet is empty; insert new Term
				stmtInTerm.setString(1, term.getTrm_tct_id());
				stmtInTerm.setString(2, term.getTrm_value());
				stmtInTerm.setString(3, term.getTrm_desc());
				
//				if (term.getTrm_edbacc() == null && term.getTrm_edb_id() == null){
//					stmtInTerm.setNull(4, java.sql.Types.VARCHAR);
//					stmtInTerm.setNull(5,  java.sql.Types.NUMERIC);
//				}else{
					stmtInTerm.setString(4, term.getTrm_edbacc());
					stmtInTerm.setString(5, term.getTrm_edb_id());
//				}
					
				logger.debug("Insert new Term. term:"+term);	
				stmtInTerm.executeUpdate();
				
				// get the generated primary key
				ResultSet rsKey = stmtInTerm.getGeneratedKeys();
				if (rsKey.next())
					term.setTrm_id(rsKey.getString(1));
				else{
					logger.fatal("synchronizeTerm: No generated primary key found");
					throw new SQLException("No generated primary key found");
				}
				
				logger.debug("Term inserted:" +term);
				
				// close rs and clear stmt
				DbUtils.closeQuietly(rsKey);
			}
			
		}catch(SQLException ex){
			throw(ex);
		}finally{
			DbUtils.closeQuietly(rs);
		
		}
		
		return term.getTrm_id();
	}
	
	/**
	 * synchronizeTerm (Insert term if not exists). <br> 
	 * insert into OBJ_PROPERTIES_T <br>
	 * 
	 * @param term The Term that should be assigned to object
	 * @param object The object (represents Entry in OBJECTS_T)
	 * @throws SQLException 
	 */
	public void insertObjProperty(Term term, AbstractPEObject object) throws SQLException{

		if (term.getTrm_id() == null)
			synchronizeTerm(term);
		
		// Set relationShip between Term and Object
		logger.debug("insert into OBJ_PROPERTIES_T  TRM_ID:"+term.getTrm_id()+" OBJ_ID:"+object.getObj_id());
		stmtInObjProp.setString(1, term.getTrm_id());
		stmtInObjProp.setString(2, object.getObj_id());
		stmtInObjProp.execute();
		
	}

	/**
	 * Checks if Objects already exists (same OBJ_Identifier, acc, edb_id)<br>
	 * If true the OBJ_ID is set.  
	 * 
	 * @param object The Objects to check.
	 * @return True if equals object exists within database and OBJ_ID was set, false otherwise.
	 * @throws SQLException
	 */
	public boolean objectExists(AbstractPEObject object) throws SQLException {
		
		ResultSet rs =null;
		try{
		
			Term idenTerm = object.getOBJ_Identifier_Term();
			
			if (idenTerm ==null || idenTerm.getTrm_edb_id() == null){
				logger.fatal("Can't check for equality. No Identifier and ExternDB ref set. object:"+object);
				return false;
			}
			
			String value = idenTerm.getTrm_value();
			String acc =  idenTerm.getTrm_edbacc();
			String edb_id = idenTerm.getTrm_edb_id();
			
			stmtQrEquealsObj.setString(1, value);
			stmtQrEquealsObj.setString(2, acc);
			stmtQrEquealsObj.setString(3, edb_id);
			rs = stmtQrEquealsObj.executeQuery();
					 
			if(rs.next()){

				String id =  rs.getString(1);
				object.setObj_id(id);
				
				if(rs.next()){
					String message = "Duplicate entries within database for duplicate:"+rs.getString(1)+"  object:"+object;
					logger.fatal(message);

					throw new SQLException(message);
				}

				logger.info("Dataset already exists. Take over OBJ_ID:"+id+" object:"+object+"");

				return true;
			}
			
		}catch( SQLException e){
			e.printStackTrace();
			logger.error("Error while searching for equals Object");
			throw(e);
		}finally{
			DbUtils.closeQuietly(rs);
		}
		
		return false;
	}	
	
		
	
	/**
	 * <ol>
	 * <li>Cecks constraints from package <code>pe_constraints</code></li>
	 * <li>Set <code>ESE_Successful</code> to 'y'.</li>
	 * <li>Closes Statements of ImportHandler.</li>
	 * </ol>
	 * @throws SQLException If SQLException occurs.
	 * @throws NullPointerException If Session is already killed or closed .
	 */
	public void closeSession() throws SQLException{
		
		if(isKilled() || isClosed()){
			String m="Can't kill session! Session already killed or closed.";
			logger.warn(m);
			throw new NullPointerException(m);
		}
		
		logger.info("Closing Session ESE_ID:"+ese.getEse_Id());
		
		final String sql="update EDITING_SESSION_T "+
					"set ESE_SUCCESSFUL ='y' "+
					"where ese_id = "+ese.getEse_Id();
		
		Statement stmt = null;
		try {
			
			checkConstrains();

			// update EDITING_SESSION_T 
			stmt = con.createStatement();
			stmt.execute(sql);	

			this.closed=true;
			
		} catch (SQLException e) {
			e.printStackTrace();
			logger.error("Error while Closing Session ESE_ID:"+ese.getEse_Id());
			throw e;
		}finally{		
			DbUtils.closeQuietly(stmt);
		}
	}//closeSession
	
	/**
	 *
	 * Deletes data of current session. <br>
	 * <br>
	 * Due to 'delete on cascade' deletes all Data from:<br>
	 * TERMS_T<br>
	 * OBJ_PROPERTIES_T<br>
	 * OBJECT_T<br>
	 * EDITING_SESSION_T <br>
	 * which are inserted during this session. <br>

	 * If successful set killed Attribut to TRUE;

	 * @throws SQLException If SQLException occurs.
	 * @throws NullPointerException If Session is already killed or closed.
	 */
	public void killSession() throws SQLException{
		
		logger.info("Killing Session ESE_ID:"+ese.getEse_Id());
		
		if(isKilled() || isClosed()){
			String m="Can't kill session! Session already killed or closed.";
			logger.warn(m);
			throw new NullPointerException(m);
		}
		
		
		final String sql = "delete from EDITING_SESSION_T where ese_id = "+this.ese.getEse_Id();
		
		Statement stmt = null;
		try {
			// Delete data
			stmt = con.createStatement();
			stmt.execute(sql);	
		
			this.killed=true;
			
		} catch (SQLException e) {
			e.printStackTrace();
			logger.error("Error while killing Session ESE_ID:"+ese.getEse_Id());
			throw e;
		}finally{
			DbUtils.closeQuietly(stmt);
			
			closeQuitly();
		}		
		
	}
	
	


	/**
	 * 
	 * @return TRUE if Seesion data were successful deleted
	 */
	public boolean isKilled() {
		return killed;
	}
	
	/**
	 * Checks constraints for the imported data <br>
	 * using the 
	 * Calls Methods of package pe_constraints
	 * @throws SQLException 
	 *
	 */
	public void checkConstrains() throws SQLException{
		
		logger.info("Checking constraints.");
		
		CallableStatement stmt = null;
		try {
			logger.info("Checking Constraints ...");
			stmt = con.prepareCall("call pe_constraints.checkConstraints(?)");
			stmt.setString(1, this.getEditingSessionID());
			stmt.execute();
			logger.info("Constraints ok.");
			
		} catch (SQLException e) {
			logger.error("checkConstrains faild");
			String m = "Error while checking constraints.";
			logger.error(m);
			e.printStackTrace();
			throw e;
		}
	}//checkConstrains

	/**
	 * @return the closed
	 */
	public boolean isClosed() {
		return closed;
	}

	
	
	/**
	 * closes the prepared statements <br>
	 * set Connection to NULL
	 *
	 */
	private void closeQuitly(){
		
		// Close all Prepared Statements
		DbUtils.closeQuietly(stmtInObject);
		DbUtils.closeQuietly(stmtInObjProp);
		DbUtils.closeQuietly(stmtInTerm);
		DbUtils.closeQuietly(stmtQrEquealsObj);
//		DbUtils.closeQuietly(stmtQrTerm);
		DbUtils.closeQuietly(stmtQrTermEDB); 
		
		this.con=null;				
	}//closeQuitly
	
	
	/**
	 * 
	 * @param ese
	 * @throws SQLException 
	 */
	private void insertEditingSession(EditingSession ese) throws SQLException{
		
		//Check if User is set
		if (ese.getEse_UserName() == null)
			ese.setEse_UserName(System.getProperty("user.name", "unknown"));
		
		logger.debug("Insert into EditingSession_T");
		
		String sql = "INSERT INTO EDITING_SESSION_T ( "+
					 "ESE_DESC, ESE_SOURCE_DESC, ESE_USERNAME) "+
					 "VALUES (?,?,?)";

		String keyColumnESE[] = {"ESE_ID"};
		PreparedStatement stmt = con.prepareStatement(sql,keyColumnESE );
		ResultSet rsKey =null;
		try {
			
			stmt.setString(1, ese.getEse_Desc());
			stmt.setString(2, ese.getEse_Source_desc());
			stmt.setString(3, ese.getEse_UserName());
			stmt.executeUpdate();
			rsKey = stmt.getGeneratedKeys();
			
			if (rsKey.next()){
				ese.setEse_Id(rsKey.getString(1));
				logger.info("EditingSession ID:"+ese.getEse_Id());
			}else{
				logger.fatal("NO ESE_ID found");
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
			logger.error("Error while insert into EditingSession_T");
			throw (e);
		}finally{
			DbUtils.closeQuietly(rsKey);
			DbUtils.closeQuietly(stmt);
		}		
	}//insertEditingSession
	
	
	/**
	 * prepaareds the required Statements
	 * @throws SQLException
	 */
	private void prepareStatements() throws SQLException{
		
		logger.info("Set AutoCommit to false !");
		con.setAutoCommit(false);
		
		logger.debug("Preparing stetements");
		
		String keyColumnObj[] = {"OBJ_ID"};
		String keyColumnTrm[] = {"TRM_ID"};
		
		logger.debug("initiating Statements");
//		stmtQrTerm = con.prepareStatement(sqlQrTerm);
		stmtQrTermEDB = con.prepareStatement(sqlQrTermEDB);
		stmtInObject = con.prepareStatement(sqlInObject, keyColumnObj);
		stmtInTerm = con.prepareStatement(sqlInTerm, keyColumnTrm);
		stmtInObjProp = con.prepareStatement(sqlInObjProp);		
		stmtQrEquealsObj = con.prepareStatement( sqlQrEquealsObj);
		
		stmtInObject.setString(2, this.ese.getEse_Id());
		stmtInTerm.setString(6, this.ese.getEse_Id());
		stmtInObjProp.setString(3, this.ese.getEse_Id());
	}//prepareStatements

	public final Connection getConnection() {
		return this.con;
	}
	
}
