package de.ipk.pdwexpress.util;

import java.sql.SQLException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.apache.commons.collections.map.MultiValueMap;
import org.apache.log4j.Logger;

import de.ipk.pdwexpress.upload.PE_Object;

public class PE_ObjectsWrapperImpl<T extends PE_Object> implements PE_ObjectsWrapper<T> {
	
	protected Logger logger;
	
	private final List<T> itemsList =new LinkedList<T>();
	
//	private Map<String, T> mapAccToT = new HashMap<String,T>();
//	private Map<String, T> mapNameToT = new HashMap<String,T>();

	private MultiValueMap mapAccToT = new MultiValueMap();
	private MultiValueMap mapNameToT = new MultiValueMap();
	
	public PE_ObjectsWrapperImpl(List<T> items) {
		logger=Logger.getLogger(this.getClass());
		
		for (T item: items){
			addItem(item);
		}
		
	}
	


	/* (non-Javadoc)
	 * @see de.ipk.pdwexpress.util.PE_ObjectsWrapper#getItems()
	 */
	public List<T> getItems() {
		// TODO nur kopie zurueckgeben
		return itemsList;
	}
	

	
	/* (non-Javadoc)
	 * @see de.ipk.pdwexpress.util.PE_ObjectsWrapper#saveAll()
	 */
	public void saveAll() throws SQLException{
		
		logger.info("Saving Objects. T:"+this.getClass().getName()+"count:"+itemsList.size());
		
		Iterator<T> it = this.itemsList.iterator();
		
		while (it.hasNext())
			 it.next().saveObject();
			
	}
	
	
	
	/* (non-Javadoc)
	 * @see de.ipk.pdwexpress.util.PE_ObjectsWrapper#getByIdentifier(java.lang.String, de.ipk.pdwexpress.upload.ExternDBInterface)
	 */
	public Collection<T> getByIdentifier(String acc){
		
		Collection<T> returnItems = mapAccToT.getCollection(acc);
		
		return returnItems;
	}
	
	/* (non-Javadoc)
	 * @see de.ipk.pdwexpress.util.PE_ObjectsWrapper#getFirstByIdentifier(java.lang.String, de.ipk.pdwexpress.upload.ExternDBInterface)
	 */
	public T getFirstByIdentifier (String acc){
		Collection<T> returnItems =getByIdentifier(acc);
		
		if (returnItems == null || returnItems.size()==0)
			return null;
		
		return returnItems.iterator().next();			
	}

	
	/* (non-Javadoc)
	 * @see de.ipk.pdwexpress.util.PE_ObjectsWrapper#getByName(java.lang.String)
	 */
	public Collection<T> getByName(String name){
		
		Collection<T> returnItems = mapNameToT.getCollection(name);
		
		return returnItems;
	}
	
	/* (non-Javadoc)
	 * @see de.ipk.pdwexpress.util.PE_ObjectsWrapper#getFirstByName(java.lang.String)
	 */
	public T getFirstByName (String name){
		Collection<T> returnItems =getByName(name);
		
		if (returnItems == null || returnItems.size()==0)
			return null;
		
		return returnItems.iterator().next();			
	}

	/**
	 * 
	 * @param item
	 */
	private void addItem(T item) {
		
		// TODO nur copy aufnehmen
		itemsList.add(item);

		String name =item.getOBJ_Name(); 
		
		if (item.getOBJ_Identifier() !=null){
			mapAccToT.put(item.getOBJ_Identifier(), item);
		}
		
		if (name != null){
			mapNameToT.put(name, item);
		}
	}
	

} // class
