package de.ipk.pdwexpress.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import org.apache.commons.dbutils.DbUtils;
import org.apache.log4j.Logger;

import de.ipk.pdwexpress.upload.PE_Object;

/**
 * Writes data into RESULT_DATA_T
 * @author Thomas
 *
 */
public class ResultDataHandler {

	public static final int MAX_STACKSIZE = 100;

	protected Logger logger;
	
	/**
	 * ResultSet ID 
	 */
	private final String rsd_obj_id_rst;
	
	/**
	 * insert statement for RESULT_DATA_T
	 */
	private PreparedStatement stmtInsert;
	
	/**
	 * Connection to PDWExpress
	 */
	private Connection con;
	
	private int stackSize=0;
	
	/**
	 * 
	 * @param con The Connection to PDWExpress
	 * @param rsd_obj_id_rst Id of the referenced ResultSet. <br>
	 * (Value for column RSD_OBJ_ID_RST in table RESULT_DATA_T.)
	 * 
	 * @throws SQLException If preparing of Statement failed.
	 */
	public ResultDataHandler(Connection con,String rsd_obj_id_rst) throws SQLException {
		logger=Logger.getLogger(this.getClass());
		
		this.rsd_obj_id_rst=rsd_obj_id_rst;
		this.con=con;
		initiate();
	}
	
	/**
	 * creates the PreparedStatement for insert
	 * @throws SQLException
	 */
	private void initiate() throws SQLException {
		logger.debug("initiate");
		stmtInsert=con.prepareStatement("INSERT INTO RESULT_DATA_T (RSD_OBJ_ID_RST, RSD_OBJ_ID, RSD_VALUE) VALUES ("+this.rsd_obj_id_rst+",?,?)");		
	}
	
	/**
	 * closes the intern PreparedStatement
	 * @throws SQLException 
	 *
	 */
	public final void close() throws SQLException{
		executeBatch();
		logger.debug("closing");
		DbUtils.closeQuietly(stmtInsert);
	}

	/**
	 * Inserts the given value into RESULT_DATA_T
	 * 
	 * @param rsd_value The value.
	 * @param rsd_obj_id The secend foreignKey in RESULT_DATA_T;
	 * @throws SQLException If insert failed.
	 */
	public void addValue(float rsd_value, String rsd_obj_id) throws SQLException{
		stmtInsert.setString(1, rsd_obj_id);
		stmtInsert.setFloat(2, rsd_value);
		
		stmtInsert.addBatch();
		stackSize++;
		if (stackSize == MAX_STACKSIZE){
			executeBatch();
		}
	}
	
	private void executeBatch() throws SQLException{
		logger.debug("stmtInsert.executeBatch()");
		stmtInsert.executeBatch();
		stackSize=0;
	}
	
	/**
	 * Calls 'addValue(float rsd_value, String rsd_obj_id)'
	 * 
	 * @param rsd_value The value.
	 * @param obj The Object assigned to the value.
	 * @throws SQLException
	 * @throws NullPointerException If obj is null.
	 */
	public void addValue(float rsd_value, PE_Object obj) throws SQLException{
		stmtInsert.setString(1, obj.getObj_id());
		stmtInsert.setFloat(2, rsd_value);
		stmtInsert.execute();
	}	

}
